@@ -687,20 +687,20 @@ sanei_hp_accessor_vector_new (HpData data, unsigned length, unsigned depth)
 }
 
 static unsigned short
-_gamma_vector_unscale (HpAccessorVector UNUSEDARG this, SANE_Fixed fval)
+_gamma_vector_unscale (HpAccessorVector __sane_unused__ this, SANE_Fixed fval)
 {
   unsigned short unscaled = fval / SANE_FIX(1.0);
   if (unscaled > 255) unscaled = 255;
-  unscaled = 255 - unscaled;  /* Dont know why. But this is how it works */
+  unscaled = 255 - unscaled;  /* Don't know why. But this is how it works. */
 
   return unscaled;
 }
 
 static SANE_Fixed
-_gamma_vector_scale (HpAccessorVector UNUSEDARG this, unsigned short val)
+_gamma_vector_scale (HpAccessorVector __sane_unused__ this, unsigned short val)
 {
   SANE_Fixed scaled;
-  val = 255-val;     /* Dont know why. But this is how it works */
+  val = 255-val;     /* Don't know why. But this is how it works. */
   scaled = val * SANE_FIX(1.0);
 
   return scaled;
