@@ -46,18 +46,12 @@
 #define HP_H_INCLUDED
 #include <limits.h>
 #include <sys/types.h>
-#include "sane/sane.h"
+#include "../include/sane/sane.h"
 
 #undef BACKEND_NAME
 #define BACKEND_NAME	hp
 #define DEBUG_NOT_STATIC
-#include "sane/sanei_debug.h"
-
-#ifdef __GNUC__
-#define UNUSEDARG __attribute__ ((unused))
-#else
-#define UNUSEDARG
-#endif
+#include "../include/sane/sanei_debug.h"
 
 /* FIXME: these should be options? */
 #undef ENABLE_7x12_TONEMAPS
@@ -70,7 +64,6 @@
 
 #define HP_CONFIG_FILE 	 STRINGIFY(BACKEND_NAME) ".conf"
 
-#define MM_PER_INCH		25.4
 #define DEVPIX_PER_INCH		300.0
 #define MM_PER_DEVPIX		(MM_PER_INCH / DEVPIX_PER_INCH)
 
@@ -105,7 +98,7 @@
   *
   */
 
-typedef int 		hp_bool_t;
+typedef unsigned int	hp_bool_t;
 typedef unsigned char	hp_byte_t;
 
 typedef enum { HP_CONNECT_SCSI, HP_CONNECT_DEVICE,
@@ -164,7 +157,7 @@ typedef	int HpScl;
 typedef long int HpScl;
 #endif
 
-void hp_init_openfd (void);
+void sanei_hp_init_openfd (void);
 
 typedef struct
 {
