@@ -1,5 +1,5 @@
 /* sane - Scanner Access Now Easy.
-   Copyright (C) 2002 Frank Zago (fzago at austin dot rr dot com)
+   Copyright (C) 2002 Frank Zago (sane at zago dot net)
 
    This file is part of the SANE package.
 
@@ -37,11 +37,11 @@
 
    If you write modifications of your own for SANE, it is your choice
    whether to permit this exception to apply to your modifications.
-   If you do not wish that, delete this exception notice. 
+   If you do not wish that, delete this exception notice.
 */
 
-/* 
-	$Id: matsushita.h,v 1.6 2003/09/12 16:10:33 hmg-guest Exp $
+/*
+	$Id$
 */
 
 /* Commands supported by the KV-SS 25 scanner. */
@@ -162,7 +162,6 @@ getbitfield (unsigned char *pageaddr, int mask, int shift)
 
 /*--------------------------------------------------------------------------*/
 
-#define MM_PER_INCH     25.4
 #define mmToIlu(mm) (((mm) * 1200) / MM_PER_INCH)
 #define iluToMm(ilu) (((ilu) * MM_PER_INCH) / 1200)
 
@@ -206,7 +205,7 @@ enum Matsushita_Option
 
 /*--------------------------------------------------------------------------*/
 
-#define BLACK_WHITE_STR		SANE_I18N("Black & White")
+#define BLACK_WHITE_STR		SANE_VALUE_SCAN_MODE_LINEART
 #define GRAY4_STR			SANE_I18N("Grayscale 4 bits")
 #define GRAY8_STR			SANE_I18N("Grayscale 8 bits")
 
@@ -229,7 +228,7 @@ SANE_I18N("Automatic separation")
 
 /*--------------------------------------------------------------------------*/
 
-/* Differences between the scanners. 
+/* Differences between the scanners.
  * The scsi_* fields are used to lookup the correcte entry. */
 struct scanners_supported
 {
@@ -343,7 +342,7 @@ Matsushita_Scanner;
 
 /*--------------------------------------------------------------------------*/
 
-/* Debug levels. 
+/* Debug levels.
  * Should be common to all backends. */
 
 #define DBG_error0  0
