@@ -39,31 +39,30 @@
    whether to permit this exception to apply to your modifications.
    If you do not wish that, delete this exception notice.  */
 
-#include "sane/config.h"
+#include "../include/sane/config.h"
 
 #include <limits.h>
 #include <stdlib.h>
 #include <string.h>
 extern int errno;
 
-#include "sane/sane.h"
-#include "sane/saneopts.h"
+#include "../include/sane/sane.h"
+#include "../include/sane/saneopts.h"
 
 #include <unistd.h>
 #include <fcntl.h>
 
-#include "sane/sanei_backend.h"
+#include "../include/sane/sanei_backend.h"
 
 #ifndef PATH_MAX
 # define PATH_MAX	1024
 #endif
 
-#include "sane/sanei_config.h"
+#include "../include/sane/sanei_config.h"
 #define PINT_CONFIG_FILE "pint.conf"
 
 #include "pint.h"
 
-#define MM_PER_INCH		25.4
 #define DECIPOINTS_PER_MM	(720.0 / MM_PER_INCH)
 #define TWELVEHUNDS_PER_MM	(1200.0 / MM_PER_INCH)
 
@@ -222,10 +221,10 @@ attach (const char *devname, PINT_Device **devp)
   if (ioctl (fd, SCIOCSET, &scanio) >= 0) \
     mode_list[lastguess ++] = modename
 
-  CHECK_MODE(SIM_BINARY_MONOCHROME, "Lineart");
-  CHECK_MODE(SIM_DITHERED_MONOCHROME, "Halftone");
-  CHECK_MODE(SIM_GRAYSCALE, "Gray");
-  CHECK_MODE(SIM_COLOR, "Color");
+  CHECK_MODE(SIM_BINARY_MONOCHROME, SANE_VALUE_SCAN_MODE_LINEART);
+  CHECK_MODE(SIM_DITHERED_MONOCHROME, SANE_VALUE_SCAN_MODE_HALFTONE);
+  CHECK_MODE(SIM_GRAYSCALE, SANE_VALUE_SCAN_MODE_GRAY);
+  CHECK_MODE(SIM_COLOR, SANE_VALUE_SCAN_MODE_COLOR);
   CHECK_MODE(SIM_RED, "Red");
   CHECK_MODE(SIM_GREEN, "Green");
   CHECK_MODE(SIM_BLUE, "Blue");
@@ -576,7 +575,7 @@ sane_init (SANE_Int *version_code, SANE_Auth_Callback authorize)
   DBG_INIT();
 
   if (version_code)
-    *version_code = SANE_VERSION_CODE (V_MAJOR, V_MINOR, 0);
+    *version_code = SANE_VERSION_CODE (SANE_CURRENT_MAJOR, V_MINOR, 0);
 
   fp = sanei_config_open (PINT_CONFIG_FILE);
   if (!fp)
@@ -860,17 +859,17 @@ sane_get_parameters (SANE_Handle handle, SANE_Parameters *params)
 
       /* set the scan image mode */
       mode = s->val[OPT_MODE].s;
-      if (!strcmp (mode, "Lineart"))
+      if (!strcmp (mode, SANE_VALUE_SCAN_MODE_LINEART))
 	{
 	  s->params.format = SANE_FRAME_GRAY;
 	  scanio.scan_image_mode = SIM_BINARY_MONOCHROME;
 	}
-      else if (!strcmp (mode, "Halftone"))
+      else if (!strcmp (mode, SANE_VALUE_SCAN_MODE_HALFTONE))
 	{
 	  s->params.format = SANE_FRAME_GRAY;
 	  scanio.scan_image_mode = SIM_DITHERED_MONOCHROME;
 	}
-      else if (!strcmp (mode, "Gray"))
+      else if (!strcmp (mode, SANE_VALUE_SCAN_MODE_GRAY))
 	{
 	  s->params.format = SANE_FRAME_GRAY;
 	  scanio.scan_image_mode = SIM_GRAYSCALE;
