@@ -7,7 +7,7 @@
  *  @brief Some I/O stuff.
  *
  * Based on sources acquired from Plustek Inc.<br>
- * Copyright (C) 2001-2005 Gerhard Jaeger <gerhard@gjaeger.de>
+ * Copyright (C) 2001-2007 Gerhard Jaeger <gerhard@gjaeger.de>
  *
  * History:
  * History:
@@ -23,9 +23,11 @@
  * - 0.47 - no changes
  * - 0.48 - cleanup
  * - 0.49 - no changes
- * - 0.50 - usbio_DetectLM983x() now returns error if register 
+ * - 0.50 - usbio_DetectLM983x() now returns error if register
  *          could not be red
  *        - usbio_ResetLM983x() checks for reg7 value before writing
+ * - 0.51 - allow dumpRegs to be called without valid fd
+ * - 0.52 - no changes
  * .
  * <hr>
  * This file is part of the SANE package.
@@ -67,9 +69,9 @@
  * If you do not wish that, delete this exception notice.
  * <hr>
  */
- 
-#include "sane/sanei_usb.h"
-#include "sane/sanei_lm983x.h"
+
+#include "../include/sane/sanei_usb.h"
+#include "../include/sane/sanei_lm983x.h"
 
 #define _UIO(func)                            \
     {                                         \
@@ -95,9 +97,10 @@ static PicDef dPix;
 
 /**
  */
-static void dumpPic( char* name, SANE_Byte *buffer, u_long len )
+static void dumpPic( char* name, SANE_Byte *buffer, u_long len, int is_gray )
 {
-	FILE *fp;
+	u_short type;
+	FILE   *fp;
 
 	if( DBG_LEVEL < _DBG_DPIC )
 		return;
@@ -112,12 +115,17 @@ static void dumpPic( char* name, SANE_Byte *buffer, u_long len )
 
 			if( 0 != dPix.x ) {
 
+				if (is_gray)
+					type = 5;
+				else
+					type = 6;
+
 				DBG( _DBG_DPIC, "> X=%lu, Y=%lu, depth=%u\n",
 				                 dPix.x, dPix.y, dPix.depth );
 				if( dPix.depth > 8 )
-					fprintf( fp, "P6\n%lu %lu\n65535\n", dPix.x, dPix.y );
+					fprintf( fp, "P%u\n%lu %lu\n65535\n", type, dPix.x, dPix.y);
     			else
-					fprintf( fp, "P6\n%lu %lu\n255\n", dPix.x, dPix.y );
+					fprintf( fp, "P%u\n%lu %lu\n255\n", type, dPix.x, dPix.y);
 			}
     	}
 	} else {
@@ -148,7 +156,10 @@ static void dumpPicInit( ScanParam *sd, char* name )
 	dPix.y	   = sd->Size.dwLines;
 	dPix.depth = sd->bBitDepth;
 
-	dumpPic( name, NULL, 0 );
+	if( sd->bDataType == SCANDATATYPE_Color )
+		dumpPic(name, NULL, 0, 0);
+	else
+		dumpPic(name, NULL, 0, 1);
 }
 
 /**
@@ -165,37 +176,38 @@ static void dumpregs( int fd, SANE_Byte *cmp )
 
 	buf[0] = '\0';
 
-	usbio_ReadReg(fd, 0x01, &regs[0x01]);
-	usbio_ReadReg(fd, 0x02, &regs[0x02]);
-	usbio_ReadReg(fd, 0x03, &regs[0x03]);
-	usbio_ReadReg(fd, 0x04, &regs[0x04]);
-	usbio_ReadReg(fd, 0x07, &regs[0x07]);
+	if( fd >= 0 ) {
+		usbio_ReadReg(fd, 0x01, &regs[0x01]);
+		usbio_ReadReg(fd, 0x02, &regs[0x02]);
+		usbio_ReadReg(fd, 0x03, &regs[0x03]);
+		usbio_ReadReg(fd, 0x04, &regs[0x04]);
+		usbio_ReadReg(fd, 0x07, &regs[0x07]);
 
-	sanei_lm983x_read( fd, 0x08, &regs[0x8], 0x80-0x8, SANE_TRUE );
+		sanei_lm983x_read( fd, 0x08, &regs[0x8], 0x80-0x8, SANE_TRUE );
 
-	for( i = 0x0; i < 0x80; i++ ) {
+		for( i = 0x0; i < 0x80; i++ ) {
 
-		if((i%16) ==0 ) {
+			if((i%16) ==0 ) {
 
-			if( buf[0] )
-				DBG( _DBG_DREGS, "%s\n", buf );
-			sprintf( buf, "0x%02x:", i );
+				if( buf[0] )
+					DBG( _DBG_DREGS, "%s\n", buf );
+				sprintf( buf, "0x%02x:", i );
+			}
+
+			if((i%8)==0)
+				strcat( buf, " ");
+
+			/* the dataport read returns with "0 Bytes read", of course. */
+			if((i == 0) || (i == 5) || (i == 6))
+				strcat( buf, "XX ");
+			else {
+
+				sprintf( b2, "%02x ", regs[i]);
+				strcat( buf, b2 );
+			}
 		}
-
-		if((i%8)==0)
-			strcat( buf, " ");
-
-		/* the dataport read returns with "0 Bytes read", of course. */
-		if((i == 0) || (i == 5) || (i == 6))
-			strcat( buf, "XX ");
-		else {
-
-			sprintf( b2, "%02x ", regs[i]);
-			strcat( buf, b2 );
-		}
-
+		DBG( _DBG_DREGS, "%s\n", buf );
 	}
-	DBG( _DBG_DREGS, "%s\n", buf );
 
 	if( cmp ) {
 
@@ -244,7 +256,7 @@ static SANE_Bool usbio_WriteReg( SANE_Int handle,
 	for( i = 0; i < 100; i++ ) {
 
 		sanei_lm983x_write_byte( handle, reg, value );
-	
+
 		/* Flush register 0x02 when register 0x58 is written */
 		if( 0x58 == reg ) {
 			_UIO( usbio_ReadReg( handle, 2, &data ));
@@ -289,7 +301,7 @@ static SANE_Status usbio_DetectLM983x( SANE_Int fd, SANE_Byte *version )
 	sprintf( buf, "usbio_DetectLM983x: found " );
 
 	switch((SANE_Int)value ) {
-		
+
 		case 4:	 strcat( buf, "LM9832/3" ); break;
 		case 3:	 strcat( buf, "LM9831" );   break;
 		case 2:	 strcat( buf, "LM9830 --> unsupported!!!" );
