@@ -2,7 +2,7 @@
    Copyright (C) 1996, 1997 Andreas Beck
    Copyright (C) 2000, 2001 Michael Herder <crapsite@gmx.net>
    Copyright (C) 2001, 2002 Henning Meier-Geinitz <henning@meier-geinitz.de>
-   Copyright (C) 2008 Stphane Voltz <stef.dev@free.fr>
+   Copyright (C) 2008 Stéphane Voltz <stef.dev@free.fr>
    This file is part of the SANE package.
 
    This program is free software; you can redistribute it and/or
@@ -106,9 +106,9 @@ static const SANE_Word resbit_list[] = {
   200, 210, 225, 240, 255, 270, 285, 300
 };
 static const SANE_Range percentage_range = {
-  -100 << SANE_FIXED_SCALE_SHIFT,	/* minimum */
-  100 << SANE_FIXED_SCALE_SHIFT,	/* maximum */
-  0 << SANE_FIXED_SCALE_SHIFT	/* quantization */
+  SANE_FIX(-100),	/* minimum */
+  SANE_FIX(100),	/* maximum */
+  SANE_FIX(0)           /* quantization */
 };
 static const SANE_Range gamma_range = {
   0,				/* minimum */
@@ -267,7 +267,7 @@ static SANE_Option_Descriptor sod[] = {
    SANE_I18N ("Hand-Scanner Simulation"),
    SANE_I18N ("Simulate a hand-scanner.  Hand-scanners often do not know the "
 	      "image height a priori.  Instead, they return a height of -1.  "
-	      "Setting this option allows to test whether a frontend can "
+	      "Setting this option allows one to test whether a frontend can "
 	      "handle this correctly."),
    SANE_TYPE_BOOL,
    SANE_UNIT_NONE,
@@ -649,7 +649,7 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
   v = 75;
 
   DBG (2, "sane_control_option: handle=%p, opt=%d, act=%d, val=%p, info=%p\n",
-       handle, option, action, value, info);
+       handle, option, action, value, (void *) info);
 
   if (handle != MAGIC || !is_open)
     {
