@@ -39,7 +39,7 @@
    If you do not wish that, delete this exception notice.  */
 
 /*
-	$Id: ricoh.h,v 1.5 2003-09-12 16:10:33 hmg-guest Exp $
+	$Id$
 */
 
 #ifndef ricoh_h
@@ -47,7 +47,7 @@
 
 #include <sys/types.h>
 
-#include <sane/config.h>
+#include "../include/sane/config.h"
 
 /* defines for scan_image_mode field */
 #define RICOH_BINARY_MONOCHROME   0
@@ -69,7 +69,9 @@
 
 static const SANE_String_Const mode_list[] =
   {
-    "Lineart", "Halftone", "Gray",
+    SANE_VALUE_SCAN_MODE_LINEART,
+    SANE_VALUE_SCAN_MODE_HALFTONE,
+    SANE_VALUE_SCAN_MODE_GRAY,
     0
   };
 
@@ -169,7 +171,7 @@ _4btol(SANE_Byte *bytes)
         return (rv);
 }
 
-typedef enum 
+typedef enum
   {
     OPT_NUM_OPTS = 0,
 
@@ -316,7 +318,7 @@ struct measurements_units_page {
 };
 
 struct mode_pages {
-        SANE_Byte page_code; 
+        SANE_Byte page_code;
         SANE_Byte parameter_length;
         SANE_Byte rest[6];
 #if 0
