@@ -1,5 +1,5 @@
 /* sane - Scanner Access Now Easy.
- 
+
    Copyright (C) 2009 m. allan noah
 
    This file is part of the SANE package.
@@ -57,6 +57,10 @@
 #ifndef SANEI_MAGIC_H
 #define SANEI_MAGIC_H
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /** Initialize sanei_magic.
  *
  * Call this before any other sanei_magic function.
@@ -141,7 +145,7 @@ sanei_magic_findEdges(SANE_Parameters * params, SANE_Byte * buffer,
  * @param buffer contains image data
  * @param top vertical offset to upper edge of crop
  * @param bot vertical offset to lower edge of crop
- * @param left horizontal offset to left edge of crop 
+ * @param left horizontal offset to left edge of crop
  * @param right horizontal offset to right edge of crop
  *
  * @return
@@ -219,4 +223,8 @@ extern SANE_Status
 sanei_magic_turn(SANE_Parameters * params, SANE_Byte * buffer,
   int angle);
 
+#ifdef __cplusplus
+} // extern "C"
+#endif
+
 #endif /* SANEI_MAGIC_H */
